<?php

class MageSage_ProductFeeds_Adminhtml_ProductfeedsController extends Mage_Adminhtml_Controller_Action
{
    private $extensionFolder;

    protected function _initAction()
    {
        $this->extensionFolder = Mage::getBaseDir('lib') . '/CartProductFeeds/core/';
        $this->loadLayout()->_setActiveMenu("productfeeds")->_addBreadcrumb(Mage::helper("productfeeds")->__("Product Feeds Manager"), Mage::helper("productfeeds")->__("Product Feeds Manager"));
        return $this;
    }

    public function indexAction()
    {
        $this->_title($this->__("Manage Feeds"));
        $this->loadLayout();
        $this->_initAction();
        $this->renderLayout();
    }

    public function createfeedAction()
    {
        $this->_initAction();
        /***********************************************************
         * //We'll display the CreateFeed Page here. CreateFeed is like a "wizard" that handles the fine points of instantiating a new feed
         * //This action is slightly different from a "newAction" because CreateFeed does a little more work than a "new"
         * /***********************************************************/

        //Step 1: Load the required files from the core
        require_once $this->extensionFolder . 'data/feedcore.php';
        require_once $this->extensionFolder . 'classes/cart-product-feed.php';
        require_once $this->extensionFolder . 'classes/dialogfeedpage.php';
        require_once $this->extensionFolder . 'classes/dialoglicensekey.php';
        require_once $this->extensionFolder . 'classes/providerlist.php';
        require_once $this->extensionFolder . 'classes/dialogfeedsettings.php';
        require_once $this->extensionFolder . 'registration.php';


        //Step 3: Render a blank page
        $this->loadLayout()->renderLayout();

    }

    public function licenseAction()
    {
        $this->loadLayout();
        $this->_title($this->__("User License"));
        $this->_initAction();
        $this->renderLayout();
    }

    public function editAction()
    {
        /***********************************************************
         * //We'll display the EditFeed Page here. EditFeed will house a specialized version of the CreateFeedPage
         * //The specialized version does not create a new feed, but rather saves the new feed over top the existing feed. This simulates an edit action
         * /***********************************************************/
        $this->_initAction();
        $this->_title($this->__("Product Feeds"));
        $this->_title($this->__("Product Feeds"));
        $this->_title($this->__("Edit Item"));

        //Step 1: Load the required files from the core. You'll want to change the folder path
        require_once $this->extensionFolder . 'classes/cart-product-feed.php';
        require_once $this->extensionFolder . 'classes/dialogeditfeed.php';
        require_once $this->extensionFolder . 'classes/dialoglicensekey.php';
        require_once $this->extensionFolder . 'classes/providerlist.php';
        require_once $this->extensionFolder . 'data/feedcore.php';
        require_once $this->extensionFolder . 'data/feedfolders.php';
        require_once $this->extensionFolder . 'data/savedfeed.php';
        require_once $this->extensionFolder . 'registration.php';

        $id = $this->getRequest()->getParam("id");
        $model = Mage::getModel("productfeeds/productfeed")->load($id);
        if ($model->getId()) {
            $this->loadLayout()->renderLayout();
        } else {
            Mage::getSingleton("adminhtml/session")->addError(Mage::helper("productfeeds")->__("Item does not exist."));
            $this->_redirect("*/*/");
        }
    }


    public function newAction()
    {
        $this->_title($this->__("Product Feeds"));
        $this->_title($this->__("Product Feeds"));
        $this->_title($this->__("New Item"));

        $id = $this->getRequest()->getParam("id");
        $model = Mage::getModel("productfeeds/productfeed")->load($id);

        $data = Mage::getSingleton("adminhtml/session")->getFormData(true);
        if (!empty($data)) {
            $model->setData($data);
        }

        Mage::register("productfeed_data", $model);
        $this->loadLayout();
        $this->_setActiveMenu("productfeeds");

        $this->getLayout()->getBlock("head")->setCanLoadExtJs(true);

        $this->_addBreadcrumb(Mage::helper("adminhtml")->__("Product Feeds Manager"), Mage::helper("adminhtml")->__("Product Feeds Manager"));

        $this->_addContent($this->getLayout()->createBlock("productfeeds/adminhtml_productfeeds_edit"))->_addLeft($this->getLayout()->createBlock("productfeeds/adminhtml_productfeeds_edit_tabs"));

        $this->renderLayout();

    }

    public function saveAction()
    {

        $post_data = $this->getRequest()->getPost();


        if ($post_data) {

            try {


                $model = Mage::getModel("productfeeds/productfeed")
                    ->addData($post_data)
                    ->setId($this->getRequest()->getParam("id"))
                    ->save();

                Mage::getSingleton("adminhtml/session")->addSuccess(Mage::helper("adminhtml")->__("Product Feed was successfully saved"));
                Mage::getSingleton("adminhtml/session")->setProductFeedData(false);

                if ($this->getRequest()->getParam("back")) {
                    $this->_redirect("*/*/edit", array("id" => $model->getId()));
                    return;
                }
                $this->_redirect("*/*/");
                return;
            } catch (Exception $e) {
                Mage::getSingleton("adminhtml/session")->addError($e->getMessage());
                Mage::getSingleton("adminhtml/session")->setProductFeedData($this->getRequest()->getPost());
                $this->_redirect("*/*/edit", array("id" => $this->getRequest()->getParam("id")));
                return;
            }

        }
        $this->_redirect("*/*/");
    }


    public function deleteAction()
    {
        if ($this->getRequest()->getParam("id") > 0) {
            try {
                $model = Mage::getModel("productfeeds/productfeed");
                $model->setId($this->getRequest()->getParam("id"))->delete();
                Mage::getSingleton("adminhtml/session")->addSuccess(Mage::helper("adminhtml")->__("Product feed was successfully deleted"));
                $this->_redirect("*/*/");
            } catch (Exception $e) {
                Mage::getSingleton("adminhtml/session")->addError($e->getMessage());
                $this->_redirect("*/*/edit", array("id" => $this->getRequest()->getParam("id")));
            }
        }
        $this->_redirect("*/*/");
    }

}