<?php

  /********************************************************************
  Version 2.0
    Magento Productlist
	  Copyright 2016. All rights reserved.
		license	GNU General Public License version 3 or later; see GPLv3.txt
	By: Keneto 2016-02
  ********************************************************************/

class PProductList {

	public $products = null;
	
	public $productStart = -1;

	//********************************************************************
	//Load the products from the DB
	//********************************************************************

	public function loadProducts($parent) {
		global $pfcore;

		//$read = Mage::getSingleton('core/resource')->getConnection('core_read');
		//$readresult = $read->query('SELECT id FROM ???');
		
		//$parent->logActivity('Reading products...');

		//???user can define how many products he wants. How to maje sure we don't get more products than he asks for?
		//if ($parent->has_product_range)
			//$limit = 'LIMIT ' . $parent->product_limit_low . ', ' . ($parent->product_limit_high - $parent->product_limit_low);

		//if ($this->productStart > -1)
			//$limit = 'LIMIT ' . $this->productStart . ', 50000';

		//???We should only fetch "visible" products

		$this->products = Mage::getModel('catalog/product')->getCollection();
		$this->products->addAttributeToSelect('*')->addAttributeToFilter('status', 1);
        if($parent->has_product_range){
            $limit = $parent->product_limit_high - $parent->product_limit_low;
            $this->products->setCurrentPage($parent->product_limit_low % $limit)->setPageSize($limit);
        }
	}

	//********************************************************************
	//Convert the ProductList
	//********************************************************************

	public function getProductList($parent, $remote_category) {

		global $pfcore;

		//********************************************************************
		//Initialize
		//********************************************************************
		$parent->logActivity('Retrieving product list from database');
		if ($this->products == null)
			$this->loadProducts($parent);

		//********************************************************************
		//Convert the Magento List into a Harmonized Product List (ListItems)
		//********************************************************************

		foreach ($this->products as $index => $product) {

			if ($index % 100 == 0)
				$parent->logActivity('Converting master product list...' . round($index / count($this->products) * 100) . '%' );

			$data = $product->getData();
            $cats = $product->getCategoryIds();
			$skip = true;
//			$cats = explode(',', $categories);
//			if (count($cats) == 0)
//				$cats[] = $prod->category;
			foreach($cats as $cat)
				if ($parent->categories->containsCategory($cat)) {
					$skip = false;
					break;
				}
			if ($skip)
				continue;

			$item = new PAProduct();
			//Basics
			$item->id = $data->entity_id; //Older method of reporting ID is rarely used in new versions of the extension
			//Attributes
			$attributes = array();
            $stockItem = Mage::getModel('cataloginventory/stock_item')->loadByProduct($product);
            $stockQty =(int) $stockItem->getQty();
			$attributes['stock_quantity'] = $stockQty; //??? $data->product_quantity;
			$attributes['title'] = $data['name'];
			$attributes['handle'] = $data['url_key'];
			$attributes['sku'] = $data['sku'];
			$attributes['product_parent_id'] = 0; //Probably Magento doesn't have parent products
			$attributes['sale_price_dates_from'] = $data['special_from_date']; //$data->product_sale_start;
			$attributes['sale_price_dates_to'] = $data['special_to_date']; //$data->product_sale_end;
			$attributes['link'] = Mage::getUrl($data['url_path']); //??? link to product that Google will give to buyer
			$attributes['valid'] = 'true';

			$attributes['feature_imgurl'] = $data['image']; //??? generate link to product image

			$item->attributes = $attributes;

			$item->attributes['regular_price'] = $data['price'];
			$item->attributes['has_sale_price'] = false;
			if (isset($data->special_price)) {
				$item->attributes['has_sale_price'] = true;
				$item->attributes['sale_price'] = $data['special_price'];
			}

			//Carry on defining product
			$item->attributes['id'] = $data['entity_id']; //This method of reporting ID is newer
			$item->taxonomy = '';
			$item->attributes['isVariable'] = false;
			$item->attributes['isVariation'] = false;
			$item->description_short = substr(strip_tags($data['short_description']), 0, 8000);
			$item->description_long = substr(strip_tags($data['description']), 0, 8000);

			//if ($data->parent_id > 0)
				//$item->attributes['item_group_id'] = $data->parent_id;

			//Fetch any default attributes Stage 0 (Mapping 3.0)
			foreach ($parent->attributeDefaults as $thisDefault)
				if ($thisDefault->stage == 0 && !$thisDefault->isRuled && !isset($item->attributes[$thisDefault->attributeName]))
					$item->attributes[$thisDefault->attributeName] = $thisDefault->getValue($item);

			//Deal with Product type... ensure it exists
			if (!isset($item->attributes['product_type']) || strlen($item->attributes['product_type']) == 0) {
				$item->attributes['product_type'] = '';
				//This code allows the product typ to come from the first matching category
				//$pcat = $parent->categories->idToCategory($cats[0]);
				//if ($pcat != null)
					//$item->attributes['product_type'] = $pcat->title;
			}

			$item->attributes['category'] = str_replace(".and.", " & ", str_replace(".in.", " > ", $item->attributes['product_type']));
			$item->attributes['localCategory'] = str_replace(".and.", " & ", str_replace(".in.", " > ", $item->attributes['product_type']));
			$item->attributes['localCategory'] = str_replace("|", ">", $item->attributes['localCategory']);

			$item->attributes['condition'] = 'New';
			$item->attributes['weight'] = $data['weight'];

			//In-stock status
			$item->attributes['stock_status'] = $stockItem->getIsInStock();
			if ($item->attributes['stock_quantity'] == 0)
				$item->attributes['stock_status'] = 0;

			//Hide out of stock
			if (($pfcore->hide_outofstock) && ($item->attributes['stock_status'] == 0))
				$item->attributes['valid'] = false;

			//Fetch any default attributes (Mapping 3.0)
			foreach ($parent->attributeDefaults as $thisDefault)
				if ($thisDefault->stage == 1 && !$thisDefault->isRuled)
					$item->attributes[$thisDefault->attributeName] = $thisDefault->getValue($item);

			$parent->handleProduct($item);
			unset($item);

		}

  }

}